"""
https://huggingface.co/incantor/aigc_cls
"""
import os
from typing import List

import numpy as np
import torch
import albumentations as A
from PIL import Image
from torch.utils.data import DataLoader
from transformers import AutoImageProcessor, AutoModelForImageClassification
from tqdm.auto import tqdm

from .base_inference import BaseImageInference, ImagePathDataset, custom_collate


class AIGCRealClassifierInference(BaseImageInference):
    """
    A model for classifying AI-generated art vs. real images using incantor/aigc_real_cls.
    Extends the BaseImageInference class for consistent usage with procslib.
    """

    def __init__(
        self,
        model_name="incantor/aigc_real_cls",
        device="cuda",
        batch_size=16,
        max_size=384,
        num_workers=8,
    ):
        super().__init__(device=device, batch_size=batch_size)
        self.model_name = model_name
        self.num_workers = num_workers

        # Albumentations transforms for resizing and padding
        self.transforms = A.Compose([
            A.LongestMaxSize(max_size=max_size),
            A.PadIfNeeded(min_height=max_size, min_width=max_size),
        ])

        self._load_model(model_name)

    def _load_model(self, checkpoint_path: str = None):
        """
        Loads the Hugging Face model + processor. 
        If checkpoint_path is not None, you could load a local checkpoint instead.
        """
        self.processor = AutoImageProcessor.from_pretrained(self.model_name)
        self.model = AutoModelForImageClassification.from_pretrained(self.model_name).to(self.device)
        self.model.eval()

    def _preprocess_image(self, pil_image: Image.Image) -> torch.Tensor:
        """
        Preprocess a single PIL image using Albumentations + HF processor.
        Returns a tensor of shape (C, H, W).
        """
        image = np.array(pil_image.convert("RGB"))  # Convert to NumPy array
        image = self.transforms(image=image)['image']  # Albumentations transforms
        pixel_values = self.processor(
            images=image,
            return_tensors="pt"
        ).pixel_values.squeeze(0)  # shape: [3, H, W]
        return pixel_values

    def _postprocess_output(self, logits: torch.Tensor):
        """
        Takes model logits (shape [B, num_classes]), argmax them, 
        then map class indices to string labels.
        Returns a list of label strings for the batch.
        """
        predicted_indices = torch.argmax(logits, dim=-1).cpu().numpy()
        id2label = self.model.config.id2label
        preds = [id2label[idx] for idx in predicted_indices]
        return preds

    def infer_many(self, image_paths: List[str], **kwargs):
        """
        Override infer_many to use pinned memory, augmentations, and a custom collate_fn.
        """
        dataset = ImagePathDataset(image_paths, preprocess_fn=self._preprocess_image)
        dataloader = DataLoader(
            dataset,
            batch_size=self.batch_size,
            num_workers=self.num_workers,
            pin_memory=True,
            collate_fn=custom_collate,
        )

        self.model.eval()
        results = []
        with torch.no_grad():
            for batch in tqdm(dataloader, desc="Inferring AI/Real classes"):
                if batch is None:
                    continue
                images, paths = batch
                images = images.to(self.device, non_blocking=True)
                outputs = self.model(images)
                batch_preds = self._postprocess_output(outputs.logits)

                for path_, pred in zip(paths, batch_preds):
                    results.append({"path": path_, "prediction": pred})

        # Convert to pandas DataFrame
        import pandas as pd
        return pd.DataFrame(results)
