# src/procslib/models/anime_real_cls.py

import torch
import torch.nn.functional as F
from PIL import Image
from transformers import AutoImageProcessor, AutoModelForImageClassification

from .base_inference import BaseImageInference

class AnimeRealClassifierInference(BaseImageInference):
    """
    Inference class for classifying images as anime or real using the Hugging Face model:
    "incantor/anime_real_cls". This class extends BaseImageInference and returns both a predicted
    label and its confidence score.
    """

    def __init__(self, model_name="incantor/anime_real_cls", device="cuda", batch_size=16, **kwargs):
        super().__init__(device=device, batch_size=batch_size, **kwargs)
        self.model_name = model_name
        self._load_model(None)

    def _load_model(self, checkpoint_path: str = None):
        """Load the model and processor from Hugging Face."""
        self.processor = AutoImageProcessor.from_pretrained(self.model_name)
        self.model = AutoModelForImageClassification.from_pretrained(self.model_name)
        self.model.to(self.device)
        self.model.eval()

    def _preprocess_image(self, pil_image: Image.Image) -> torch.Tensor:
        """
        Preprocess a single PIL image into the format expected by the model.
        Returns a torch.Tensor of shape (C, H, W).
        """
        inputs = self.processor(images=pil_image, return_tensors="pt")
        # inputs.pixel_values has shape [1, 3, H, W]; remove the batch dimension
        return inputs.pixel_values.squeeze(0)

    def _postprocess_output(self, logits: torch.Tensor):
        """
        Postprocess raw logits into a list of dictionaries. For each image in the batch, compute the
        softmax probabilities, select the predicted label, and include the corresponding confidence score.
        Returns a list of dictionaries like:
            {"label": <str>, "confidence": <float>}
        """
        probabilities = F.softmax(logits, dim=-1)
        predicted_class_idx = torch.argmax(logits, dim=-1)
        confidence_scores = probabilities.max(dim=-1).values

        # Retrieve label mapping from model config
        id2label = self.model.config.id2label
        predictions = []
        for idx, conf in zip(predicted_class_idx, confidence_scores):
            label = id2label[idx.item()]
            predictions.append({"label": label, "confidence": float(conf)})
        return predictions
