"""
classifying image categories using HuggingFace model;

Availabe categories:
  1. Animal
  2. Art
  3. character
  4. City
  5. Food
  6. Illustration
  7. Indoor
  8. Plant
  9. Product
  10. Scenery
  11. transportation
"""

# src/procslib/models/szh_image_category_cls.py

import torch
from transformers import AutoImageProcessor, AutoModelForImageClassification
from PIL import Image

from .base_inference import BaseImageInference


class SzhImageCategoryInference(BaseImageInference):
    """
    A model for classifying image categories using HuggingFace model: szh/image_category_cls
    Extends the BaseImageInference class for consistency with procslib usage.
    """

    def __init__(self, model_name="incantor/image_category_cls", device="cuda", batch_size=16, **kwargs):
        """
        Args:
            model_name (str): The Hugging Face model identifier.
            device (str): 'cuda' or 'cpu'.
            batch_size (int): Default batch size for DataLoader.
        """
        super().__init__(device=device, batch_size=batch_size)
        self.model_name = model_name
        self._load_model(self.model_name)

    def _load_model(self, checkpoint_path: str = None):
        """
        Load the model and processor from a Hugging Face pretrained checkpoint.
        If you had a local checkpoint, you could modify this to load from disk.
        """
        self.processor = AutoImageProcessor.from_pretrained(self.model_name)
        self.model = AutoModelForImageClassification.from_pretrained(self.model_name)
        self.model.to(self.device)
        self.model.eval()

    def _preprocess_image(self, pil_image: Image.Image) -> torch.Tensor:
        """
        Preprocess a single PIL image using the model's AutoImageProcessor.
        Returns a tensor of shape (C, H, W).
        """
        # The processor returns a dict like {'pixel_values': <tensor>}, shape [1, 3, H, W]
        pixel_values = self.processor(images=pil_image, return_tensors="pt").pixel_values
        return pixel_values.squeeze(0)  # shape becomes [3, H, W]

    def _postprocess_output(self, logits: torch.Tensor):
        """
        Takes model logits (shape [B, num_classes]), does argmax, 
        and converts indices to string labels using id2label.
        Returns a list of label strings for the batch.
        """
        predicted_indices = torch.argmax(logits, dim=-1).cpu().numpy()
        # id2label is a dict: e.g. {0: 'cat', 1: 'dog', ...}
        id2label = self.model.config.id2label
        preds = [id2label[idx] for idx in predicted_indices]
        return preds
